// Copyright 2000-2024 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.lang.java.lexer;

import com.intellij.lexer.Lexer;
import com.intellij.lexer.MergeFunction;
import com.intellij.lexer.MergingLexerAdapterBase;
import com.intellij.psi.TokenType;
import com.intellij.psi.tree.IElementType;

/**
 * Lexer to wrap around the Java Lexer when parsing Java types for the markdown JavaDoc (JEP-467)
 * In reference links, array types are escaped:  "char\[\]" which the JavaLexer doesn't like
 * <p>
 * It does this by <em>covering up</em> a <i>BAD_CHARACTER</i> token if followed by <i>[</i> or <i>]</i>
 */
public class JavaTypeEscapeLexer extends MergingLexerAdapterBase {
  private final MergeFunction myMergeFunction = new EscapeMarkdownFunction();

  public JavaTypeEscapeLexer(BasicJavaLexer original) {
    super(original);
  }

  @Override
  public MergeFunction getMergeFunction() {
    return myMergeFunction;
  }


  private static class EscapeMarkdownFunction implements MergeFunction {
    @Override
    public IElementType merge(IElementType type, Lexer originalLexer) {
      if(type != TokenType.BAD_CHARACTER) return type;

      CharSequence tokenText = originalLexer.getTokenSequence();
      if (tokenText.length() != 1 || (tokenText.charAt(0) != '[' && tokenText.charAt(0) != ']')) {
        return type;
      }

      type = originalLexer.getTokenType();
      originalLexer.advance();
      return type;
    }
  }
}
